@@ -7,14 +7,15 @@ use utf8;
 
 BEGIN {
 	$MooX::Struct::AUTHORITY = 'cpan:TOBYINK';
-	$MooX::Struct::VERSION   = '0.018';
+	$MooX::Struct::VERSION   = '0.019';
 }
 
 use Moo 1.000;
 use Types::TypeTiny 1.000 qw( HashLike ArrayLike );
-use Types::Standard 1.000 qw( HashRef ArrayRef Num Ref );
+use Types::Standard 1.000 qw( HashRef ArrayRef Num Ref InstanceOf );
+use namespace::autoclean;
 
-my $HashLike  = HashLike | Ref['HASH'];
+my $HashLike  = HashLike  | Ref['HASH'];
 my $ArrayLike = ArrayLike | Ref['ARRAY'];
 
 use overload
@@ -34,7 +35,12 @@ METHODS: {
 	sub TO_STRING   { join q[ ], @{ $_[0]->TO_ARRAY } };
 	sub CLONE       { my $s = shift; ref($s)->new(%{$s->TO_HASH}, @_) };
 	sub CLASSNAME   { ref($_[0]) or $_[0] };
-	sub TYPE_TINY   { Types::Standard::InstanceOf->parameterize(shift->CLASSNAME) };
+	
+	my %_cache;
+	sub TYPE_TINY {
+		my $class = shift->CLASSNAME;
+		$_cache{$class} ||= (InstanceOf[$class])->plus_constructors(HashRef|ArrayRef, 'new');
+	}
 };
 
 sub BUILDARGS
@@ -137,7 +143,7 @@ BEGIN {
 	{
 		no warnings;
 		our $AUTHORITY = 'cpan:TOBYINK';
-		our $VERSION   = '0.018';
+		our $VERSION   = '0.019';
 	}
 	
 	sub _uniq { my %seen; grep { not $seen{$_}++ } @_ };
@@ -460,7 +466,6 @@ sub import
 	"$class\::Processor"->new->process($caller, @_);
 }
 
-no Moo;
 1;
 
 __END__
@@ -677,8 +682,8 @@ suitable for a Moose/Moo C<isa>.
    use Moo;
    use MooX::Struct Bar => [qw( $name )];
    
-   has left_bar  => (is => 'rw', isa => Bar->TYPE_TINY);
-   has right_bar => (is => 'rw', isa => Bar->TYPE_TINY);
+   has left_bar  => (is => 'rw', isa => Bar->TYPE_TINY, coerce => 1);
+   has right_bar => (is => 'rw', isa => Bar->TYPE_TINY, coerce => 1);
    
    ...;
  }
